// A demonstration of the visual effect of deceleration:
// your eye is better able to track a moving object that
// accelerates and decelerates, and you can anticipate
// the part of the motion that hasn't happened yet.

// This isn't the same as the animation principle of 
// anticipation, which I talked about at the end of
// Module 04.  Hmm, perhaps "Prediction" would have been
// a better name for this sketch.

// I intend only to show this code on the screen as a 
// demonstration.  You don't need to understand how it works,
// but it couldn't hurt to give you a copy in case you want
// to play with it.

float t;
float goal;
float gt;

boolean decel;
int launch_time;

void setup()
{
  size( 800, 100 );
  decel = false;
  launch_time = millis();
  goal = random( 100, 800 );
  gt = goal * 1000.0 / 400.0; // In milliseconds
}

void draw()
{
  background( 220 );
  
  // Use an "implicit" simulation method: calculate
  // position based on the time in the external world,
  // not the frame count.
  
  float tm = millis() - launch_time;
  
  if( tm > gt ) {
    tm = gt;
  }
  
  float pos = 0.0;
  
  if( decel ) {
    float A = -goal / sq( gt );
    float B = -2*A*gt;
    pos = A*tm*tm + B*tm;
  } else {
    pos = lerp( 0, goal, tm / gt );
  }
  
  ellipse( pos, 50, 50, 50 );
}

void keyPressed()
{
  if( key == ' ' ) {
    // Space bar restarts the simulation with a new random
    // target distance.
    launch_time = millis();
    goal = random( 100, 800 );
    gt = goal * 1000.0 / 400.0; // In milliseconds
  } else if( key == 'd' ) {
    // 'd' toggles deceleration.
    decel = !decel;
  }
}
