// Implement a chunk of a classic Breakout game, just to see
// how far we can push Fisica.

import fisica.*;

// Some obvious global variables for things we'll want to keep
// track of in the sketch.

FWorld world;
FCircle ball;
FCircle paddle;
FBox[] bricks;

void setup()
{
  size( 800, 400 );
  Fisica.init( this );
  world = new FWorld();
  
  world.setEdges();
  world.setEdgesFriction( 0 );

  // Take away the bottom edge of the screen, so that you
  // can lose the ball.
  world.remove( world.bottom );
  // No gravity in this world.
  world.setGravity( 0, 0 );
  
  // The size of a brick.
  float h = 20;
  float w = 50;
  
  bricks = new FBox[ 60 ];
  int count = 0;
  
  // Generate the bricks.
  for( int row = 0; row < 5; ++row ) {
    // Random colour for all the bricks in each row.
    color c = color( random(255), random(255), random(255) );
    float y = 50 + row*h;
    for( int col = 0; col < 12; ++col ) {
      float x = 125 + col*50;
      FBox box = new FBox( w, h );
      box.setPosition( x, y );
      box.setFill( red(c), green(c), blue(c) );
      
      // Bricks are static objects (they don't fly around
      // in the simulation) and they're not grabbable.
      box.setStatic( true );
      box.setGrabbable( false );
      // Here's the clever bit that I'm happy with: by
      // setting the restitution to a number slightly higher
      // than 1, every time the ball hits a brick is speeds
      // up a bit.
      box.setRestitution( 1.01 );
      box.setFriction( 0 );
      world.add( box );
      
      bricks[count] = box;
      ++count;
    }
  }
  
  // Create the paddle, which will look like a dome sitting
  // at the bottom of the screen.  Again, give it a restitution
  // slightly higher than 1.
  paddle = new FCircle( 200 );
  paddle.setNoStroke();
  paddle.setFill( 80 );
  paddle.setPosition( width/2, height+50 );
  paddle.setStatic( true );
  paddle.setGrabbable( false );
  paddle.setRestitution( 1.05 );
  paddle.setFriction( 0 );
  world.add( paddle );
  
  ball = null;
}

void makeBall()
{
  ball = new FCircle( 20 );
  ball.setNoStroke();
  ball.setFill( 0 );
  ball.setRestitution( 1.0 );
  ball.setPosition( width/2, 300 );
  ball.setVelocity( random( -50, 50 ), random( 200, 300  ) );
  ball.setDamping( 0 );
  // Setting the "bullet" property apparently makes a body
  // use more computation in the simulation, but become more
  // precise.  I don't see a difference, but I figured I'd
  // try it.
  ball.setBullet( true );
  ball.setFriction( 0 );
  world.add( ball );
}

void draw()
{
  background( 255 );
  
  // Did the user lose the ball?
  if( (ball != null) && (ball.getY() > height + 10) ) {
      world.remove( ball );
      ball = null;
  }
  
  // Move the paddle based on the mouse position.
  paddle.setPosition( constrain( mouseX, 50, 750 ), height+50 );
  
  world.step();
  world.draw();
}

// A bit of programming hackery.  Look for a given body
// in the list of bricks.  If you find it, do some work
// to remove that element from the array.
void checkBrick( FBody bod )
{
  for( int idx = 0; idx < bricks.length; ++idx ) {
    if( bricks[idx] == bod ) {
      world.remove( bod );
      if( idx < (bricks.length-1) ) {
        bricks[idx] = bricks[bricks.length-1];
      }
      bricks = (FBox[])shorten( bricks );
    }
  }
}

void contactResult( FContactResult con )
{
  FBody other = null;
  if( con.getBody1() == ball ) {
    other = con.getBody2();
  } else if( con.getBody2() == ball ) {
    other = con.getBody1();
  }
  
  checkBrick( other );
}

void keyPressed()
{
  if( ball == null ) {
    makeBall();
  }
}
