// A demonstration of a damped spring: the spring does the
// usual harmonic oscillation, but loses a tiny fraction of
// its energy at every time step.  As a result it will eventually
// stop oscillating and sit at its rest position.

float position = 800;
float speed = 0.0;

// The rest length of the spring,
// relative to the left edge of the
// sketch.
float rest_length = 400.0;
// Constant in Hooke's law
float stiffness = 0.003;
// Proportion of speed lost
float damping = 0.01; 

void setup()
{
  size( 800, 100 );
}

// I'll do this in a couple of steps, for clarity.
// Normally I might compress the simulation into fewer
// lines of code.
void step()
{
  // How far is the spring extended from its
  // rest length? This is a signed value, i.e.,
  // an arrow that always points away from 
  // the rest length.
  float disp = position - rest_length;
  // F = -kx -bv
  float accel = -stiffness*disp - damping*speed;

  // Accelerate towards rest length
  speed = speed + accel;
    
  // NOTE: in lecture I presented the simulation this way:
  // float accel = -stiffness * disp;
  // speed = speed + accel;
  // speed = speed * (1-damping)
  // Apparently this isn't the correct formula, and you 
  // should use the "F=-kx-bv" formula above.  But if you do
  // the math, you see that the difference will be tiny, so
  // it probably doesn't matter that much.
  
  
  // Now update position
  position = position + speed;
}

void draw()
{
  // Run the physical simulation for
  // one step
  step();
  
  // Draw the world
  background( 80 );
  ellipse( position, 50, 50, 50 );
}
