// For fun, a quick demonstration I cooked up that combines
// direct manipulation from Module 03 with sliding friction.
// When the user releases the mouse, we allow the circle to
// continue in its current direction until it slides to a stop.

float damping = 0.1;

// Position
float x = 250;
float y = 250;
// Velocity
float vx = 0;
float vy = 0;

// Are we currently dragging?
boolean dragging;

void setup()
{
  size( 500, 500 );
  dragging = false;
}

void draw()
{
  background( 80 );

  if ( dragging ) {
    ellipse( mouseX, mouseY, 50, 50 );
  } else {
    // Run a small sliding friction simulation until the
    // circle stops moving.
    
    // A bit redundant if the object has already come to a stop,
    // but we'll let it go for this simple demo.  
    vx *= (1-damping);
    vy *= (1-damping);
    
    // When the speed gets low enough, manually drop it 
    // to zero so it doesn't keep nudging forward by
    // tiny amounts.
    if ( mag( vx, vy ) < 0.1 ) {
      vx = 0.0;
      vy = 0.0;
    }

    x += vx;
    y += vy;

    ellipse( x, y, 50, 50 );
  }
}

void mousePressed()
{
  if ( dist( mouseX, mouseY, x, y ) < 25 ) {
    dragging = true;
  }
}

void mouseReleased()
{  
  if ( dragging ) {
    dragging = false;
    vx = mouseX - pmouseX;
    vy = mouseY - pmouseY;
    x = mouseX;
    y = mouseY;
  }
}
