// For fun, a quick demonstration I cooked up that combines
// direct manipulation from Module 03 with sliding friction.
// When the user releases the mouse, we allow the circle to
// continue in its current direction until it slides to a stop.

// This version applies the "correct" simulation of sliding
// friction to bring the object to a stop.

float friction = 1;

// Position
float x = 250;
float y = 250;
// Velocity
float vx = 0;
float vy = 0;

// Are we currently dragging?
boolean dragging;

void setup()
{
  size( 500, 500 );
  dragging = false;
}

void draw()
{
  background( 80 );

  if ( dragging ) {
    ellipse( mouseX, mouseY, 50, 50 );
  } else {
    // Run a small sliding friction simulation until the
    // circle stops moving.
    
    // Use the Pythagorean theorem to get the total speed.
    float av = mag( vx, vy );
    
    // Here's a nice trick for cases where sliding friction
    // is the only force affecting velocity.  We need to check
    // whether the current speed is sufficiently low that 
    // friction will bring the object to a complete stop. 
    // We check that very simply by comparing the total
    // speed against the actual friction amount.
    if( av < friction ) {
      // Speed not high enough to overcome friction, so set
      // velocity to 0.  If you don't do this, the object 
      // will reverse direction!
      vx = 0;
      vy = 0;
    } else {
      // This is the same as saying 
      // vx = vx - (vx/av) * friction;
      vx *= (1-friction/av);
      vy *= (1-friction/av);
    }

    x += vx;
    y += vy;

    ellipse( x, y, 50, 50 );
  }
}

void mousePressed()
{
  if ( dist( mouseX, mouseY, x, y ) < 25 ) {
    dragging = true;
  }
}

void mouseReleased()
{  
  if ( dragging ) {
    dragging = false;
    vx = mouseX - pmouseX;
    vy = mouseY - pmouseY;
    x = mouseX;
    y = mouseY;
  }
}

void keyPressed()
{
  x = width/2;
  y = width/2;
  vx = 0;
  vy = 0;
}
