// A fairly long sketch that accumulates a bunch of 
// easing functions so that you can compare them visually.
// The most interesting part for the purposes of this course
// is the look at the different easing functions, especially
// the simple ones.

// Press a key to turn the circle around and send it back
// in the direction it was coming from.  Drag the red circles
// to adjust the start and end of the motion.  Choose an 
// easing function using the radio buttons at the bottom.

import controlP5.*;

ControlP5 cp5;

float startX = 100.0;
float startY = 100.0;
float endX = 700.0;
float endY = 500.0;
int dragging;

float x = 100;
float y = 100;

boolean up;
float t = 0;

int ease_mode = 1;

void setup()
{
  size( 800, 650 );

  cp5 = new ControlP5( this );
  cp5.addRadioButton( "Function" )
    .setPosition( 20, 610 )
      .setSize( 30, 30 )
        .setItemsPerRow( 6 )
          .setSpacingColumn( 80 )
            .addItem( "Linear", 1 )
              .addItem( "Cubic", 2 )
                .addItem( "Quintic", 3 )
                  .addItem( "Cosine", 4 )
                    .addItem( "Anticipate", 5 )
                      .addItem( "Bounce", 6 )
                        .activate( 0 );

  up = false;
  dragging = -1;
}

// Some of these are taken from
// http://www.imaginary-institute.com/resources/TechNote04/TechNote04.html
float ease( float t )
{
  if ( ease_mode == 2 ) {
    return t*t*(-2*t+3);
  } else if ( ease_mode == 3 ) {
    return t*t*t*(6*t*t-15*t+10);
  } else if ( ease_mode == 4 ) {
    return (cos(PI*t) - 1.0) / -2.0;
  } else if ( ease_mode == 5 ) {
    if ( t < 0.05 ) {
      return -t;
    } else {
      float t2 = 1.0 - (t-0.05)/0.95;
      float v = 1.0 - t2*t2*t2;
      return lerp( -0.05, 1.0, v );
    }
  } else if ( ease_mode == 6 ) {
    float switchCenter = 0.6;
    float blendRadius = 0.05;
    float sinHeight = 0.45;
    float blendStart = switchCenter-blendRadius;
    float firstPart = pow( t/switchCenter, 3.0 );
    if (t < blendStart) {
      return firstPart;
    } else {
      float t2 = (t-blendStart)/(1-blendStart);
      float theta = 2 * (2*PI) * t2;
      float scl = pow(2, -7*t2) - 0.0078125;
      float secondPart = 1 + sinHeight * scl * sin(theta);
      float blendEnd = switchCenter+blendRadius;
      if (t > blendEnd) {
        return secondPart;
      } else {
        float blend = (t-blendStart)/(2*blendRadius);
        blend = blend*blend*(-2*blend+3);
        return lerp( firstPart, secondPart, blend );
      }
    }
  } else {
    return t;
  }
}

void draw()
{
  background( 80 );

  // Warp the current time to carry out easing.
  
  float my_t = t;

  if ( up ) {
    // This looks like the general easing framework I
    // presented in class.
    my_t = ease( t );
  } else {
    // This is basically the same formula, but I reversed
    // everything so that the object will move back from
    // point B to point A.
    my_t = 1.0 - ease( 1.0 - t );
  }

  // Draw the two direct manipulation handles for
  // the start and end of the motion.
  fill( #700000 );
  ellipse( startX, startY, 50, 50 );
  ellipse( endX, endY, 50, 50 );

  // Draw the actual moving circle.
  fill( #ffffff );
  ellipse(
    lerp( startX, endX, my_t ), 
    lerp( startY, endY, my_t ), 
    100, 100 );

  // Update the current time.  Maybe go up, maybe
  // go down.
  if ( up ) {
    t = min( 1.0, t + 0.01 );
  } else {
    t = max( 0.0, t - 0.01 );
  }
}

void keyPressed()
{
  // Reverse interpolation direction.
  up = !up;
}

void controlEvent( ControlEvent ev )
{
  ease_mode = int( ev.getGroup().getValue() );
}

void mousePressed()
{
  dragging = -1;
  if( dist( mouseX, mouseY, startX, startY ) < 25 ) {
    dragging = 0;
  } else if( dist( mouseX, mouseY, endX, endY ) < 25 ) {
    dragging = 1;
  }
}

void mouseDragged()
{
  if( dragging == 0 ) {
    startX = mouseX;
    startY = mouseY;
  } else if( dragging == 1 ) {
    endX = mouseX;
    endY = mouseY;
  } 
}

void mouseReleased()
{
  dragging = -1;
}
