// Demonstrate the power of watching for collisions.  Create
// an object that changes colour and loses "hit points" when
// it collides with other objects.  When the hit points go
// to zero, simulate the object "exploding".

import fisica.*;

FWorld world;

FBody circ;
float pts;
float sc = 0.0001;

void setup()
{
  size( 500, 500 );

  Fisica.init( this );
  world = new FWorld();
  world.setEdges();
 // world.setGravity( 0, 0 );

  circ = new FCircle( 100 );
  circ.setPosition( width/2, height/2 );
  circ.setFill( 0, 255, 0 );
  circ.setNoStroke();
  world.add( circ );

  pts = 100;
  
  // Hmm, you probably don't have the "Gotham" font on your
  // computer.  You may need to change this to another font.
  textFont( createFont( "Gotham", 36 ) );
}

void draw()
{
  background( 100 );

  float t = pts / 100.0;

  // Set the object's colour based on hit points.
  if( circ != null ) {
    circ.setFill( lerp(255, 0, t), lerp(0, 255, t), 0 );
  }
  
  // Do Fisica stuff
  world.step();
  world.draw();
  
  // Write the number of hit points.
  fill( 255 );
  float ipts = int(pts*10) / 10.0;
  text( "" + ipts, 30, 50 );
}

// If there's a collision, keep track of the object's
// health.
void contactResult( FContactResult con )
{
  if ( circ != null ) {
    // println( con.getNormalImpulse() * sc );
    pts -= con.getNormalImpulse() * sc;

    if ( pts <= 0 ) {
      // Explosion!
      float x = circ.getX();
      float y = circ.getY();
      float vx = circ.getVelocityX();
      float vy = circ.getVelocityY();
      
      for( int idx = 0; idx < 24; ++idx ) {
        FPoly poly = new FPoly();
        float a1 = TWO_PI * idx / 24.0;
        float a2 = TWO_PI * (idx+1) / 24.0;
        poly.vertex( x, y );
        poly.vertex( x+50*cos(a1), y+50*sin(a1) );
        poly.vertex( x+50*cos(a2), y+50*sin(a2) );
        poly.setFill( 255, 0, 0 );
        poly.setNoStroke();
        float vel = random( 10, 500 );
        poly.setVelocity( vx+vel*cos(a1), vy+vel*sin(a1) );
        world.add( poly );
      }
      world.remove( circ );
      circ = null;
    }
  }
}

