// A demonstration of a sliding object losing energy
// to friction, sliding to a stop.

// This version applies the "correct" simulation of sliding
// friction to bring the object to a stop.

float px = 50;
float py = 50;

float vx = 10.0;
float vy = 7.0;

// Energy loss due to friction
float friction = 0.2; 

void setup()
{
  size( 600, 600 );
}

void step()
{
  // Lose some speed due to friction

  // Here's a nice trick for cases where sliding friction
  // is the only force affecting velocity.  We need to check
  // whether the current speed is sufficiently low that 
  // friction will bring the object to a complete stop. 
  // We check that very simply by comparing the total
  // speed against the actual friction amount.
  float av = mag( vx, vy );
  if ( av < friction ) {
    // Speed not high enough to overcome friction, so set
    // velocity to 0.  If you don't do this, the object 
    // will reverse direction!
    vx = 0;
    vy = 0;
  } else {
    // This is the same as saying 
    // vx = vx - (vx/av) * friction;
    vx *= (1-friction/av);
    vy *= (1-friction/av);
  }

  // Update position using speed
  px = px + vx;
  py = py + vy;
}

void draw()
{  
  // Run the physical simulation for
  // one step
  step();

  // Draw the world
  background( 80 );
  ellipse( px, py, 50, 50 );
}

