// A quick hack that adds squash and stretch to 
// a bouncing ball.  As with the "DirectManipDamping"
// sketch, I wouldn't expect you to be able to come 
// up with an implementation like this in this course.
// But I don't mind showing it to you in case you're
// interesting in trying to do something similar.

// Standard Second Law stuff.
float position = 0;
float speed = 0;
float accel = 0.4;

// horizontal scaling factor.  We'll *compress* the
// vertical dimension by the same factor.
float squetch = 1.0;

// Hacking around to track the squash phase of a 
// bounce.
int bounce_frames;
float max_squash = 1.5;
float pos_in;
float vel_in;
float squetch_in;

void setup()
{
  size( 100, 600 );
  bounce_frames = 0;
}

void step()
{
  if ( bounce_frames > 0 ) {
    float t = 11 - bounce_frames;

    if( t <= 5 ) {
      squetch = map( t, 0, 5, squetch_in, max_squash );
    } else {
       squetch = map( t, 5, 10, max_squash, squetch_in );
    }
    position = height - 1/squetch;
      
    --bounce_frames;
    if( bounce_frames == 0 ) {
      speed = -vel_in;
      position = pos_in;
      squetch = map( abs(speed), 0, 20, 1.0, 0.66 );
    }
  } else {
    // Update speed using acceleration.
    speed = speed + accel;
    // Update position using speed.
    position = position + speed;

    // When your absolute speed is high, stretch out
    // vertically.
    squetch = map( abs(speed), 0, 20, 1.0, 0.66 );

    // Take stretch into account when detecting collision.
    if ( position+25/squetch > height ) {
      pos_in = position;
      vel_in = speed;
      squetch_in = squetch;
      bounce_frames = 11;
    }
  }
}

void draw()
{
  // Run the physical simulation for
  // one step
  step();

  // Draw the world
  background( 80 );

  ellipse( 50, position, 50*squetch, 50/squetch );
}

