// This is a quick attempt to put together a more physically
// realistic version of the ThirdLaw sketch.  This is advanced --
// you don't need to understand how it works for this course.
// But I included it because the inaccuracy in the simpler
// ThirdLaw sketch was bothering me, and I wanted to figure out
// how to correct it.

float position = 0;
float speed = 0;
float accel = 0.8;

void setup()
{
  size( 100, 600 );
}

void step()
{
  speed += accel;
  
  // Is the bottom of the circle going to collide with
  // the bottom of the sketch in the next time step?
  if( (position+25) + speed > height ) {
    // There will be a collision in this time step. 
    // Get accurate collision information.
    
    // How far we have left to move before the
    // bottom of the circle touches the bottom of
    // the sketch.
    float d = height - (position+25);
    
    // Solve for exactly how much time will pass until 
    // the exact moment of the collision, measured as a 
    // fraction of a frame.  I worked the formula out
    // with a bit of calculus, and the result requires
    // a quick use of the quadratic formula.  Uh, yay?
    float tc = (-speed + sqrt( speed*speed + 2*accel*d )) / accel;
    // Figure out what the exact speed will be at the
    // moment of collision.
    // println( tc );
    float vc = speed + tc*accel;
    
    // At this point we can force the ball to the exact
    // bottom of the sketch with the correct speed, and it
    // won't lose any energy.  You could stop after these
    // two lines and get a sketch with bounces that don't
    // decay......
    position = height - 25;
    speed = -vc;
    
    // ...but that's not quite accurate, because the ball should
    // have a brief interval in which to start climbing again 
    // before the next frame.  So push the simulation forward
    // by the remaining fraction of a frame (i.e., 1-tc time
    // units).  There may never be an actual frame drawn in
    // which the ball is resting on the floor, but overall the
    // simulation will look accurate.
    speed += accel*(1-tc);
    position += speed*(1-tc);
  } else {
    // No collision -- don't need to do anything special.
    position += speed;
  }
}

void draw()
{
  // Run the physical simulation for
  // one step
  step();
  
  // Draw the world
  background( 80 );
  ellipse( 50, position, 50, 50 );
}
